export const LANGUAGES = [
    ['af-ZA', gettext('Afrikaans'), 'ltr'],
    ['sq-AL', gettext('Albanian'), 'ltr'],
    ['ar', gettext('Arabic'), 'rtl'],
    ['ast', gettext('Asturian'), 'ltr'],
    ['be', gettext('Belarusian'), 'ltr'],
    ['br', gettext('Breton'), 'ltr'],
    ['bg', gettext('Bulgarian'), 'ltr'],
    ['ca', gettext('Catalan'), 'ltr'],
    ['ca-ES-Valencia', gettext('Catalan (Valencia)'), 'ltr'],
    ['zh-CN', gettext('Chinese (Simplified)'), 'ltr'],
    ['da', gettext('Danish'), 'ltr'],
    ['nl', gettext('Dutch'), 'ltr'],
    ['en-AU', gettext('English (Australia)'), 'ltr'],
    ['en-CA', gettext('English (Canada)'), 'ltr'],
    ['en-NZ', gettext('English (New Zealand)'), 'ltr'],
    ['en-ZA', gettext('English (South Africa)'), 'ltr'],
    ['en-GB', gettext('English (United Kingdom)'), 'ltr'],
    ['en-US', gettext('English (United States)'), 'ltr'],
    ['eo', gettext('Esperanto'), 'ltr'],
    ['fr', gettext('French'), 'ltr'],
    ['gl', gettext('Galician'), 'ltr'],
    ['de-DE', gettext('German (Germany)'), 'ltr'],
    ['de-AU', gettext('German (Austria)'), 'ltr'],
    ['de-CH', gettext('German (Switzerland)'), 'ltr'],
    ['el', gettext('Greek'), 'ltr'],
    ['he', gettext('Hebrew'), 'rtl'],
    ['is', gettext('Icelandic'), 'ltr'],
    ['it', gettext('Italian'), 'ltr'],
    ['ja', gettext('Japanese'), 'ltr'],
    ['km', gettext('Khmer'), 'ltr'],
    ['lt', gettext('Lithuanian'), 'ltr'],
    ['ml', gettext('Malayalam'), 'ltr'],
    ['nb-NO', gettext('Norwegian (bokmål)'), 'ltr'],
    ['nn-NO', gettext('Norwegian (nynorsk)'), 'ltr'],
    ['fa', gettext('Persian'), 'rtl'],
    ['pl', gettext('Polish'), 'ltr'],
    ['pt-BR', gettext('Portuguese (Brazil)'), 'ltr'],
    ['pt-PT', gettext('Portuguese (Portugal)'), 'ltr'],
    ['ro', gettext('Romanian'), 'ltr'],
    ['ru', gettext('Russian'), 'ltr'],
    ['tr', gettext('Turkish'), 'ltr'],
    ['sr-SP-Cy', gettext('Serbian (Cyrillic)'), 'ltr'],
    ['sr-SP-Lt', gettext('Serbian (Latin)'), 'ltr'],
    ['sk', gettext('Slovak'), 'ltr'],
    ['sl', gettext('Slovenian'), 'ltr'],
    ['es', gettext('Spanish'), 'ltr'],
    ['sv', gettext('Swedish'), 'ltr'],
    ['ta', gettext('Tamil'), 'ltr'],
    ['tl', gettext('Tagalog'), 'ltr'],
    ['uk', gettext('Ukrainian'), 'ltr']
]

export const PAPER_SIZES = [
    /* [Fidus Writer value, CSS value]*/
    ['A4', 'A4'],
    ['US Letter', 'letter']
]
