{% extends "admin/change_form.html" %}
{% load i18n %}
{% load transpile %}
{% block extrahead %}{{ block.super }}
<!-- Font Awesome -->
<link type="text/css" rel="stylesheet" href="{% static "css/fontawesome/css/all.css" %}" />
<script type="text/javascript" src="{% url 'admin:jsi18n' %}"></script>
<!--  Translations -->
<script type="text/javascript" src="/api/jsi18n/"></script>
<!-- JS error handling -->
<script type="text/javascript" src="{% static "js/error_hook.mjs" %}" crossorigin="anonymous"></script>
<script type="text/javascript" src="{% static "js/document_template_admin.mjs" %}"></script>
{{ media }}
{% endblock %}
