# Generated by Django 3.1 on 2020-08-20 03:10

from django.db import migrations, models
import usermedia.models


class Migration(migrations.Migration):

    dependencies = [
        ('usermedia', '0005_move_json_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documentimage',
            name='copyright',
            field=models.JSONField(default=usermedia.models.default_copyright),
        ),
        migrations.AlterField(
            model_name='userimage',
            name='copyright',
            field=models.JSONField(default=usermedia.models.default_copyright),
        ),
    ]
